package pr_r;
import java.io.*;
import java.net.*;

public class R08_Server extends Thread {
    public final static int DEFAULT_PORT = 6789;
    protected int port;
    protected ServerSocket listen_socket;

    // Exit with an error message, when an exception occurs.
    public static void fail(Exception e, String msg) {
        System.err.println(msg + ": " +  e);
        System.exit(1);
    }

    // Create a ServerSocket to listen for connections on;  start the thread.
    public R08_Server(int port) {
        if (port == 0) port = DEFAULT_PORT;
        this.port = port;
        try { listen_socket = new ServerSocket(port); }
        catch (IOException e) { fail(e, "when creating server socket"); }
        System.out.println("Server: listening on port " + port);
        this.start();
    }

    // The server is thread looping forever, listening for and
    // accepting connections from clients.  For each connection,
    // create a Clerk object to handle communication through the new Socket.
    public void run() {
        try {
            while(true) {
                Socket client_socket = listen_socket.accept();
                Clerk c = new Clerk(client_socket);
            }
        } catch (IOException e) { fail(e, "while listening for connections");}
    }

    // Start the server up, listening on an optionally specified port
    public static void main(String[] args) {
        int port = 0;
        if (args.length == 1) {
            try { port = Integer.parseInt(args[0]);  }
            catch (NumberFormatException e) { port = 0; }
        }
        System.out.println("port="+port);
        new R08_Server(port);
    }


// This class is the thread that handles all communication with a client
class Clerk extends Thread {
    protected Socket client;
    protected BufferedReader in;
    protected PrintWriter out;

    // Initialize the streams and start the thread
    public Clerk(Socket client_socket) {
        client = client_socket;
        try {
            in = new BufferedReader
            (new InputStreamReader(client.getInputStream()));
//            in  = new DataInputStream(client.getInputStream());
            out = new PrintWriter(client.getOutputStream(),true /* autoFlush */ );
        }
        catch (IOException e) {
            try { client.close(); } catch (IOException e2) { ; }
            System.err.println("Exception while getting socket streams: " + e);
            return;
        }
        this.start();
    }

    // Provide the service: Read a line, reverse it, send it back.
    public void run() {
        String line;
        StringBuffer revline;
        int len;
        try {
            for(;;) {
                line = in.readLine();   // read in a line
                if (line == null) break;
                len = line.length();
                revline = new StringBuffer(len);
                for(int i = len-1; i >= 0; i--)
                    revline.insert(len-1-i, line.charAt(i));
                out.println(revline); // write reversed line
                System.out.println("client"+client+": "+line+"->"+revline);
            }
        }
        catch (IOException e) { ; }
        finally { try {client.close();} catch (IOException e2) {;} }
    }
}
}